<?php

/**
 * Nette Framework
 *
 * Copyright (c) 2004, 2009 David Grudl (http://davidgrudl.com)
 *
 * This source file is subject to the "Nette license" that is bundled
 * with this package in the file license.txt.
 *
 * For more information please see http://nettephp.com
 *
 * @copyright  Copyright (c) 2004, 2009 David Grudl
 * @license    http://nettephp.com/license  Nette license
 * @link       http://nettephp.com
 * @category   Nette
 * @package    Forms
 */



require_once dirname(__FILE__) . '/../../Object.php';



/**
 * NUser validation JavaScript generator.
 *
 * @author     David Grudl
 * @copyright  Copyright (c) 2004, 2009 David Grudl
 * @package    Forms
 */
class NUserClientScript extends NObject
{
	/** @var NForm */
	private $form;



	public function __construct(NForm $form)
	{
		$this->form = $form;
	}



	/**
	 * Generates the client side validation script.
	 * @return void
	 */
	public function renderClientScript()
	{
		$this->form->getElementPrototype()->attrs['data-nette-rules'] = json_encode($this->exportContainer($this->form));
	}



	/**
	 * Exports description for JavaScript.
	 * @return array
	 */
	public function exportContainer(NFormContainer $container)
	{
		$data = array();
		foreach ($container->getComponents() as $name => $control) {
			if ($control instanceof NFormContainer) {
				$data[$name] = $this->exportContainer($control);

			} elseif ($control instanceof IFormControl) {
				$data[$name] = $this->exportControl($control);
			}
		}
		return array(
			'class' => $container->getClass(),
			'controls' => $data,
		);
	}



	/**
	 * Exports description for JavaScript.
	 * @return array
	 */
	private function exportControl(IFormControl $control)
	{
		return $control->isDisabled() ? NULL : array(
			'class' => $control->getClass(),
			'rules' => $this->exportRules($control->getRules()),
			'opt' => $control instanceof NFormControl ? $control->getOptions() : NULL,
			'scope' => $control instanceof ISubmitterControl ? (bool) $control->getValidationScope() : NULL,
		);
	}



	/**
	 * Exports rules for JavaScript.
	 * @return array
	 */
	private function exportRules(NRules $rules)
	{
		$data = array();
		foreach ($rules as $rule) {
			if (!is_string($rule->operation)) continue;
			$data[] = array(
				'class' => $rule->control->getClass(),
				'op' => $rule->operation,
				'neg' => $rule->isNegative,
				'type' => $rule->type,
				'msg' => $rule->message,
				'id' => $rule->control->getHtmlId(),
				'arg' => $rule->arg instanceof NFormControl ? $rule->arg->getHtmlId() : $rule->arg,
				'sub' => $rule->subRules ? $this->exportRules($rule->subRules) : NULL,
			);
		}
		return $data;
		/*return array(
			'rules' => $data,
			'toggles' => $this->toggles,
		);*/
	}

}
