<?php

/**
 * Nette Framework
 *
 * Copyright (c) 2004, 2009 David Grudl (http://davidgrudl.com)
 *
 * This source file is subject to the "Nette license" that is bundled
 * with this package in the file license.txt.
 *
 * For more information please see http://nettephp.com
 *
 * @copyright  Copyright (c) 2004, 2009 David Grudl
 * @license    http://nettephp.com/license  Nette license
 * @link       http://nettephp.com
 * @category   Nette
 * @package    Templates
 */



require_once dirname(__FILE__) . '/../Templates/BaseTemplate.php';

require_once dirname(__FILE__) . '/../Templates/IFileTemplate.php';



/**
 * NTemplate stored in file.
 *
 * @author     David Grudl
 * @copyright  Copyright (c) 2004, 2009 David Grudl
 * @package    Templates
 */
class NTemplate extends NBaseTemplate implements IFileTemplate
{
	/** @var int */
	public static $cacheExpire = FALSE;

	/** @var ICacheStorage */
	private static $cacheStorage;

	/** @var string */
	private $file;



	/**
	 * Sets the path to the template file.
	 * @param  string  template file path
	 * @return NTemplate  provides a fluent interface
	 */
	public function setFile($file)
	{
		$this->file = $file;
		return $this;
	}



	/**
	 * Returns the path to the template file.
	 * @return string  template file path
	 */
	public function getFile()
	{
		return $this->file;
	}



	/********************* rendering ****************d*g**/



	/**
	 * Renders template to output.
	 * @return void
	 */
	public function render()
	{
		if ($this->file == NULL) { // intentionally ==
			throw new InvalidStateException("NTemplate file name was not specified.");

		} elseif (!is_file($this->file) || !is_readable($this->file)) {
			throw new FileNotFoundException("Missing template file '$this->file'.");
		}

		$this->__set('template', $this);

		$cache = new NCache($this->getCacheStorage(), 'Nette.NTemplate');
		$key = md5($this->file) . '.' . basename($this->file);
		$cached = $content = $cache[$key];

		if ($content === NULL) {
			if (!$this->getFilters()) {
				$this->onPrepareFilters($this);
			}

			if (!$this->getFilters()) {
				NLimitedScope::load($this->file, $this->getParams());
				return;
			}

			try {
				$shortName = $this->file;
				$shortName = str_replace(NEnvironment::getVariable('templatesDir'), "\xE2\x80\xA6", $shortName);
			} catch (Exception $foo) {
			}

			$content = $this->compile(file_get_contents($this->file), "file $shortName");
			$cache->save(
				$key,
				$content,
				array(
					NCache::FILES => $this->file,
					NCache::EXPIRE => self::$cacheExpire,
				)
			);
			$cached = $cache[$key];
		}

		if ($cached !== NULL && self::$cacheStorage instanceof NTemplateCacheStorage) {
			NLimitedScope::load($cached['file'], $this->getParams());
			fclose($cached['handle']);

		} else {
			NLimitedScope::evaluate($content, $this->getParams());
		}
	}



	/********************* caching ****************d*g**/



	/**
	 * NSet cache storage.
	 * @param  NCache
	 * @return void
	 */
	public static function setCacheStorage(ICacheStorage $storage)
	{
		self::$cacheStorage = $storage;
	}



	/**
	 * @return ICacheStorage
	 */
	public static function getCacheStorage()
	{
		if (self::$cacheStorage === NULL) {
			self::$cacheStorage = new NTemplateCacheStorage(NEnvironment::getVariable('tempDir'));
		}
		return self::$cacheStorage;
	}

}
